@@ -10,6 +10,7 @@ from Globals import MessageDialog, DTMLFile
 
 from AccessControl import ClassSecurityInfo
 from AccessControl import Permissions
+from AccessControl import getSecurityManager
 from Globals import InitializeClass
 from Acquisition import Implicit, aq_inner, aq_parent, aq_base
 from Globals import Persistent
@@ -166,9 +167,9 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
         ret = self._getGroups(no_recurse = 1)
         return list(ret)
     
-    def _getGroups(self, no_recurse = 0, already_done = [], prefix = GROUP_PREFIX):
+    def _getGroups(self, no_recurse = 0, already_done = None, prefix = GROUP_PREFIX):
         """
-        getGroups(self, no_recurse = 0, already_done = [], prefix = GROUP_PREFIX) => list of strings
+        getGroups(self, no_recurse = 0, already_done = None, prefix = GROUP_PREFIX) => list of strings
         
         If this user is a user (uh, uh), get its groups.
         THIS METHODS NOW SUPPORTS NESTED GROUPS ! :-)
@@ -179,6 +180,9 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
 
         This method is private and should remain so.
         """
+        if already_done is None:
+            already_done = []
+
         # List this user's roles. We consider that roles starting 
         # with GROUP_PREFIX are in fact groups, and thus are 
         # returned (prefixed).
@@ -337,6 +341,7 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
 
         inner_obj = getattr(object, 'aq_inner', object)
         while 1:
+            # Usual local roles retreiving
             local_roles = getattr(inner_obj, '__ac_local_roles__', None)
             if local_roles:
                 if callable(local_roles):
@@ -351,7 +356,13 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
                 for groupid in user_groups:
                     for role in dict.get(groupid, []):
                         roles[role] = 1
+                        
+            # LocalRole blocking
+            obj = getattr(inner_obj, 'aq_base', inner_obj)
+            if getattr(obj, '__ac_local_roles_block__', None):
+                break
 
+            # Loop management
             inner = getattr(inner_obj, 'aq_inner', inner_obj)
             parent = getattr(inner, 'aq_parent', None)
             if parent is not None:
@@ -372,13 +383,14 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
 
 
     security.declarePrivate("getProperty")
-    def getProperty(self, name, default = _marker):
+    def getProperty(self, name, default=_marker):
         """getProperty(self, name) => return property value or raise AttributeError
         """
         # Try to do an attribute lookup on the underlying user object
-        v = getattr(self.__underlying__, name, _marker)
+        v = getattr(self.__underlying__, name, default)
         if v is _marker:
             raise AttributeError, name
+        return v
 
     security.declarePrivate("hasProperty")
     def hasProperty(self, name):
@@ -425,7 +437,7 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
         v = self._GRUF.getUserById(self.getId()).getProperty(name)
         if not v == value:
             Log(LOG_DEBUG, "Property '%s' for user '%s' should be '%s' and not '%s'" % (
-                name, self.getId(), v, value,
+                name, self.getId(), value, v,
                 ))
             raise NotImplementedError, "Property setting is not supported for '%s'." % (name,)
 
@@ -513,6 +525,12 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
                             return 1
                         return None
 
+            # LocalRole blocking
+            obj = getattr(inner_obj, 'aq_base', inner_obj)
+            if getattr(obj, '__ac_local_roles_block__', None):
+                break
+
+            # Loop control
             inner = getattr(inner_obj, 'aq_inner', inner_obj)
             parent = getattr(inner, 'aq_parent', None)
             if parent is not None:
@@ -583,7 +601,7 @@ class GRUFUserAtom(AccessControl.User.BasicUser, Implicit):
 
         ret = '''<a href="%(href)s" alt="%(alt)s"><font color="%(color)s">%(name)s</font></a>''' % {
             "color": color,
-            "href": "%s/%s/manage_workspace" % (acl_users.absolute_url(), self.getId(), ),
+            "href": "%s/%s/manage_workspace?FORCE_USER=1" % (acl_users.absolute_url(), self.getId(), ),
             "name": self.getUserNameWithoutGroupPrefix(),
             "alt": "%s (%s)" % (self.getUserNameWithoutGroupPrefix(), kind, ),
             }
@@ -617,9 +635,18 @@ class GRUFUser(GRUFUserAtom):
     #                     User Mutation                         #
     #                                                           #
 
-    security.declarePrivate('changePassword')
+    security.declarePublic('changePassword')
     def changePassword(self, password):
-        """Set the user's password"""
+        """Set the user's password. This method performs its own security checks"""
+        # Check security
+        user = getSecurityManager().getUser()
+        if not user.has_permission(Permissions.manage_users, self._GRUF):       # Is manager ?
+            if user.__class__.__name__ != "GRUFUser":
+                raise "Unauthorized", "You cannot change someone else's password."
+            if not user.getId() == self.getId():    # Is myself ?
+                raise "Unauthorized", "You cannot change someone else's password."
+        
+        # Just do it
         self.clearCachedGroupsAndRoles()
         return self._GRUF.userSetPassword(self.getId(), password)
 
@@ -739,8 +766,15 @@ class GRUFGroup(GRUFUserAtom):
 
         # Extraction
         groupid = self.getId()
-        return [u for u in lst
-                if groupid in getattr(gruf.getUser(u), method)()]
+        groups_mapping = {}
+        for u in lst:
+            usr = gruf.getUser(u)
+            if not usr:
+                groups_mapping[u] = []
+                Log(LOG_WARNING, "Invalid user retreiving:", u)
+            else:
+                groups_mapping[u] = getattr(usr, method)()
+        return [u for u in lst if groupid in groups_mapping[u]]
 
     security.declarePrivate("getMemberIds")
     def getMemberIds(self, transitive = 1, ):
