@@ -5,7 +5,7 @@
 ##############################################################################
 """ Basic usergroup tool.
 
-$Id: GroupsTool.py,v 1.30 2004/06/15 10:04:43 pjgrizel Exp $
+$Id: GroupsTool.py,v 1.36 2005/02/04 09:15:06 pjgrizel Exp $
 """
 
 from Products.CMFCore.utils import UniqueObject
@@ -28,8 +28,11 @@ from Products.CMFCore.ActionProviderBase import ActionProviderBase
 from interfaces.portal_groups import portal_groups as IGroupsTool
 from global_symbols import *
 
+# Optional feature-preview support
+import PloneFeaturePreview
 
-class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
+
+class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase, ):
     """ This tool accesses group data through a GRUF acl_users object.
 
     It can be replaced with something that groups member data in a
@@ -62,9 +65,9 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                      },
                 ) + SimpleItem.manage_options)
 
-    #
-    #   ZMI methods
-    #
+    #                                                   #
+    #                   ZMI methods                     #
+    #                                                   #
     security.declareProtected(ViewManagementScreens, 'manage_overview')
     manage_overview = DTMLFile('dtml/explainGroupsTool', globals())     # unlike MembershipTool
     security.declareProtected(ViewManagementScreens, 'manage_config')
@@ -102,7 +105,7 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
 
     security.declareProtected(ViewGroups, 'getGroupsByUserId')
     def getGroupsByUserId(self, userid):
-        """Returns a list of the groups the user corresponding to 'userid' belongs to."""
+        """Return a list of the groups the user corresponding to 'userid' belongs to."""
         #log("getGroupsByUserId(%s)" % userid)
         user = self.acl_users.getUser(userid)
         #log("user '%s' is in groups %s" % (userid, user.getGroups()))
@@ -114,17 +117,17 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
 
     security.declareProtected(ViewGroups, 'listGroups')
     def listGroups(self):
-        """Returns a list of the available portal_groupdata-ish objects."""
+        """Return a list of the available portal_groupdata-ish objects."""
         return [self.wrapGroup(elt) for elt in self.acl_users.getGroups()]
 
     security.declareProtected(ViewGroups, 'listGroupIds')
     def listGroupIds(self):
-        """Returns a list of the available groups' ids as entered (without group prefixes)."""
+        """Return a list of the available groups' ids as entered (without group prefixes)."""
         return self.acl_users.getGroupNames()
 
     security.declareProtected(ViewGroups, 'listGroupNames')
     def listGroupNames(self):
-        """Returns a list of the available groups' ids as entered (without group prefixes)."""
+        """Return a list of the available groups' ids as entered (without group prefixes)."""
         return self.acl_users.getGroupNames()
 
     security.declarePublic("isGroup")
@@ -215,13 +218,11 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         if id in self.listGroupIds():
             raise ValueError, "Group '%s' already exists." % (id, )
         self.acl_users.userFolderAddGroup(id, roles = roles, groups = groups )
-        Log(LOG_DEBUG, "In addGroup()")
         self.createGrouparea(id)
-        Log(LOG_DEBUG, "In addGroup()")
         self.getGroupById(id).setProperties(**kw)
 
     security.declareProtected(ManageGroups, 'editGroup')
-    def editGroup(self, id, roles = [], groups = [], *args, **kw):
+    def editGroup(self, id, roles = None, groups = None, *args, **kw):
         """Edit the given group with the supplied password, roles, and domains.
 
         Underlying user folder must support editing users via the usual Zope API.
@@ -321,17 +322,18 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
     def createGrouparea(self, id):
         """Create a space in the portal for the given group, much like member home
         folders."""
-        Log(LOG_DEBUG, )
         parent = self.aq_inner.aq_parent
         workspaces = self.getGroupWorkspacesFolder()
         pt = getToolByName( self, 'portal_types' )
 
-        Log(LOG_DEBUG, )
         if id and self.getGroupWorkspacesCreationFlag():
             if workspaces is None:
                 # add GroupWorkspaces folder
-
-                parent.invokeFactory(self.getGroupWorkspaceContainerType(), self.getGroupWorkspacesFolderId())
+                pt.constructContent(
+                    type_name = self.getGroupWorkspaceContainerType(),
+                    container = parent,
+                    id = self.getGroupWorkspacesFolderId(),
+                    )
                 workspaces = self.getGroupWorkspacesFolder()
                 workspaces.setTitle(self.getGroupWorkspacesFolderTitle())
                 workspaces.setDescription("Container for " + self.getGroupWorkspacesFolderId())
@@ -341,15 +343,15 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                 portal_catalog = getToolByName( self, 'portal_catalog' )
                 portal_catalog.unindexObject(workspaces)     # unindex GroupWorkspaces folder
                 workspaces._setProperty('right_slots', (), 'lines')
-
-            Log(LOG_DEBUG, )
+                
             if workspaces is not None and not hasattr(workspaces, id):
                 # add workspace to GroupWorkspaces folder
-                Log(LOG_DEBUG, self.getGroupWorkspaceType(), id)
-                workspaces.invokeFactory(self.getGroupWorkspaceType(), id)
-                Log(LOG_DEBUG, )
+                pt.constructContent(
+                    type_name = self.getGroupWorkspaceType(),
+                    container = workspaces,
+                    id = id,
+                    )
                 space = self.getGroupareaFolder(id)
-                Log(LOG_DEBUG, )
                 space.setTitle("%s workspace" % id)
                 space.setDescription("Container for objects shared by this group")
 
@@ -362,6 +364,13 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                 else:
                     space.manage_delLocalRoles(space.users_with_local_role('Owner'))
                     self.setGroupOwnership(self.getGroupById(id), space)
+
+                # Hook to allow doing other things after grouparea creation.
+                notify_script = getattr(workspaces, 'notifyGroupAreaCreated', None)
+                if notify_script is not None:
+                    notify_script()
+
+                # Re-indexation
                 portal_catalog = getToolByName( self, 'portal_catalog' )
                 portal_catalog.reindexObject(space)
  
@@ -468,4 +477,6 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         # Failed.
         return g
 
+
+
 InitializeClass(GroupsTool)
