@@ -41,7 +41,6 @@ class LDAPGroupFolder(SimpleItem):
     def __setstate__(self, v):
         """ """
         LDAPGroupFolder.inheritedAttribute('__setstate__')(self, v)
-
         self._cache = SimpleCache()
         self._cache.setTimeout(600)
         self._cache.clear()
@@ -143,6 +142,11 @@ class LDAPGroupFolder(SimpleItem):
     security.declareProtected(manage_users, 'getUser')
     def getUser(self, name):
         """ """
+        # Prevent locally stored groups
+        luf = self.getLUF()
+        if luf._local_groups:
+            return []
+
         # Get the group from the cache
         user = self._cache.get(name, '')
         if user:
@@ -186,11 +190,22 @@ class LDAPGroupFolder(SimpleItem):
     security.declareProtected(manage_users, 'getUserNames')
     def getUserNames(self):
         """ """
-        return [g[0] for g in self.getLUF().getGroups()]
+        Log(LOG_DEBUG, "getUserNames", )
+        LogCallStack(LOG_DEBUG)
+        # Prevent locally stored groups
+        luf = self.getLUF()
+        if luf._local_groups:
+            return []
+        return [g[0] for g in luf.getGroups()]
 
     security.declareProtected(manage_users, 'getUsers')
     def getUsers(self, authenticated=1):
         """ """
+        # Prevent locally stored groups
+        luf = self.getLUF()
+        if luf._local_groups:
+            return []
+
         data = []
         
         grps = self.getLUF().getGroups()
@@ -223,6 +238,7 @@ class LDAPGroupFolder(SimpleItem):
         with this system, it's possible to differenciate between LDAP groups and LDAP roles.
         """
         self.getLUF().manage_addGroup(name)
+        self.manage_addGroupMapping(name, "group_" + name, None, )
         self._doChangeUser(name, password, roles, domains, **kw)
 
     security.declarePrivate('_doDelUsers')
@@ -237,10 +253,15 @@ class LDAPGroupFolder(SimpleItem):
 
     security.declarePrivate('_doChangeUser')
     def _doChangeUser(self, name, password, roles, domains, **kw):
-        """WARNING: If a role with exists with the same name as the group, we do not add
-        the group mapping for it, but we create it as if it were a Zope ROLE.
-        Ie. it's not possible to have a GRUF Group name = a Zope role name, BUT,
-        with this system, it's possible to differenciate between LDAP groups and LDAP roles.
+        """
+        This is used to change the groups (especially their roles).
+
+        [ THIS TEXT IS OUTDATED :
+          WARNING: If a ZOPE role with the same name as the GRUF group exists,
+          we do not add the group mapping for it, but we create it as if it were a Zope ROLE.
+          Ie. it's not possible to have a GRUF Group name = a Zope role name, BUT,
+          with this system, it's possible to differenciate between LDAP groups and LDAP roles.
+        ]
         """
         luf = self.getLUF()
         self._cache.remove(name)
