@@ -72,17 +72,28 @@ class IUserFolder(Interface):
 
     def getUserById(id, default):
         """Return the user atom corresponding to the given id.
-        If default is provided, return default if no user found, else raise an exception
+        If default is provided, return default if no user found, else return None.
         """
 
     def getUserByName(name, default):
         """Same as getUserById() but works with a name instead of an id.
-        If default is provided, return default if no user found, else raise an exception
+        If default is provided, return default if no user found, else return None.
         [NOTA: Theorically, the id is a handle, while the name is the actual login name.
         But difference between a user id and a user name is unsignificant in
         all current User Folder implementations... except for GROUPS.]        
         """
 
+    def hasUsers():
+        """
+        From Zope 2.7's User.py:
+        This is not a formal API method: it is used only to provide
+        a way for the quickstart page to determine if the default user
+        folder contains any users to provide instructions on how to
+        add a user for newbies.  Using getUserNames or getUsers would have
+        posed a denial of service risk.
+        In GRUF, this method always return 1."""
+    
+
     # Search interface for users; they won't return groups in any case.
 
     def searchUsersByName(search_term):
@@ -105,6 +116,7 @@ class IUserFolder(Interface):
         THIS METHOD MAY BE VERY EXPENSIVE ON USER FOLDER KINDS WHICH DO NOT PROVIDE A
         SEARCHING METHOD (ie. every UF kind except LDAPUF).
         'attribute' can be 'id' or 'name' for all UF kinds, or anything else for LDAPUF.
+        [NOTA: This method is time-expensive !]
         """
 
     # Search interface for groups;
@@ -129,6 +141,7 @@ class IUserFolder(Interface):
         THIS METHOD MAY BE VERY EXPENSIVE ON GROUP FOLDER KINDS WHICH DO NOT PROVIDE A
         SEARCHING METHOD (ie. every UF kind except LDAPUF).
         'attribute' can be 'id' or 'name' for all UF kinds, or anything else for LDAPUF.
+        [NOTA: This method is time-expensive !]
         """
 
 
@@ -198,6 +211,11 @@ class IUserFolder(Interface):
         attributes.
         Groups can be specified by name or by id (preferabily by name)."""
 
+    def userFolderUpdateUser(name, password, roles, domains, groups, **kw):
+        """Same as userFolderEditUser, but with all arguments except name
+        being optional.
+        """
+
     def userFolderDelUsers(names):
         """API method for deleting one or more user atom objects. Note that not
         all user folder implementations support deletion of user objects."""
@@ -210,6 +228,11 @@ class IUserFolder(Interface):
         """API method for changing group object attributes.
         """
 
+    def userFolderUpdateGroup(name, roles, groups, **kw):
+        """Same as userFolderEditGroup but with all arguments (except name) being
+        optinal.
+        """
+
     def userFolderDelGroups(names):
         """API method for deleting one or more group objects.
         Implem. note : All ids must be prefixed with 'group_',
@@ -348,6 +371,25 @@ class IUserFolder(Interface):
         """Same as listMemberUserIds but only return group ids.
         THIS METHOD CAN BE VERY EXPENSIVE AS IT NEEDS TO FETCH ALL USERS.
         """
+
+
+    # Local roles acquisition blocking support
+    def acquireLocalRoles(folder, status):
+        """Enable or disable local role acquisition on the specified folder.
+        If status is true, it will enable, else it will disable.
+        """
+
+    def isLocalRoleAcquired(folder):
+        """Return true if the specified folder allows local role acquisition.
+        """
+
+    # Audit & security checking methods
+
+    def getAllLocalRoles(object):
+        """getAllLocalRoles(self, object): return a dictionnary {user: roles} of local
+        roles defined AND herited at a certain point. This will handle lr-blocking
+        as well.
+        """
         
 
 class IUserAtom(Interface):
