@@ -17,7 +17,9 @@ from Testing import ZopeTestCase
 from AccessControl.Permissions import access_contents_information, view, add_documents_images_and_files, change_images_and_files, view_management_screens
 from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager, getSecurityManager
 from AccessControl import Unauthorized
+from AccessControl import Permissions
 from AccessControl.User import UnrestrictedUser
+from AccessControl.PermissionRole import rolesForPermissionOn
 
 import urllib
 
@@ -116,6 +118,7 @@ class TestGroupUserFolder(GRUFTestCase.GRUFTestCase):
             password = 'secret',
             roles = ('userrole', ),
             groups = ('gtest', ),
+            domains = (),
             )
 
         # Check if the user has the right roles
@@ -180,15 +183,43 @@ class TestGroupUserFolder(GRUFTestCase.GRUFTestCase):
 
         We just check that people has the right roles
         """
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u1", ()))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u2", ("r3", )))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u3", ("r1", "r3", )))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u4", ("r1", "r2", "r3", )))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u5", ("r1", "r2", )))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u6", ("r1", "r2", "r3", )))
-        self.failUnless(self.compareRoles(self.gruf_folder.lr, "u7", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u1", ()))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u2", ()))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u3", ("r1", )))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u4", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u5", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u6", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.gruf_folder, "u7", ("r1", "r2", "r3", )))
 
+        self.failUnless(self.compareRoles(self.lr, "u1", ()))
+        self.failUnless(self.compareRoles(self.lr, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.lr, "u3", ("r1", "r3", )))
+        self.failUnless(self.compareRoles(self.lr, "u4", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.lr, "u5", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.lr, "u6", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.lr, "u7", ("r1", "r2", "r3", )))
 
+        self.failUnless(self.compareRoles(self.sublr, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.sublr, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.sublr, "u6", ("r1", "r2", "r3", )))
+
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", "r3", )))
+
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", "r3", )))
+
+        self.failUnless(self.compareRoles(self.sublr3, "u2", ()))
+        self.failUnless(self.compareRoles(self.sublr3, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.sublr3, "u6", ("r1", "r2", )))
+
+        self.failUnless(self.compareRoles(self.subsublr3, "u2", ()))
+        self.failUnless(self.compareRoles(self.subsublr3, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr3, "u6", ("r1", "r2", )))
+
+        
     def test05nestedGroups(self,):
         """
         Test security on nested groups
@@ -252,62 +283,163 @@ class TestGroupUserFolder(GRUFTestCase.GRUFTestCase):
         #urllib.urlopen(base+'/acl_users/getGRUFId')
 
 
-
     #                                                   #
-    #              Classical security testing           #
+    #           LocalRole Acquisition Blocking          #
     #                                                   #
 
-##    def testAccess(self):
-##        '''Test access'''
-##        page = self.gruf_folder(self.gruf_folder)
+    def test11LocalRoleBlocking(self,):
+        """
+        We block LR acquisition on sublr2.
+        See GRUFTestCase to understand what happens (basically, roles in brackets
+        will be removed from sublr2).
+        """
+        # Initial check
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", "r3", )))
+        
+        # Disable LR acquisition on sublr2 and test the stuff
+        self.gruf._acquireLocalRoles(self.sublr2, 0)
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", )))
 
-##    def testWeb(self):
-##        '''Test web access'''
-##        urllib._urlopener = GRUFTestCase.UnauthorizedOpener()
-##        Log(LOG_DEBUG, base)
-##        page = urllib.urlopen(base).read()
-##        if page.find('Resource not found') >= 0:
-##            self.fail('Resource not found')
+        # Now we disable LR acq. on subsublr2 and check what happens
+        self.gruf._acquireLocalRoles(self.subsublr2, 0)
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", )))
 
-##    def testWeb2(self):
-##        '''Test web access to protected resource'''
-##        urllib._urlopener = GRUFTestCase.ManagementOpener()
-##        page = urllib.urlopen(base+'/secret_html').read()
-##        if page.find('Resource not found') >= 0:
-##            self.fail('Resource not found')
+        # We enable back on sublr2. subsublr2 mustn't change.
+        self.gruf._acquireLocalRoles(self.sublr2, 1)
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", )))
 
-##    def testSecurity(self):
-##        '''Test security of public resource'''
-##        # Should be accessible
-##        try: self.gruf_folder.restrictedTraverse('index_html')
-##        except Unauthorized: self.fail('Unauthorized')
+    def test12LocalRoleSecurity(self):
+        """Access TTW
+        """
+        try:
+            self.gruf.acquireLocalRoles(self.sublr2, 1)
+        except:
+            failed = 1
+        else:
+            failed = 0
 
-##    def testSecurity2(self):
-##        '''Test security of protected resource'''
-##        # Should be protected
-##        self.assertRaises(Unauthorized, self.gruf_folder.restrictedTraverse, 'secret_html')
-
-##    def testWebSecurity(self):
-##        '''Test web security of public resource'''
-##        # Should be accessible
-##        urllib._urlopener = GRUFTestCase.UnauthorizedOpener()
-##        try: urllib.urlopen(base+'/index_html')
-##        except Unauthorized: self.fail('Unauthorized')
-
-##    def testWebSecurity2(self):
-##        '''Test web security of protected resource'''
-##        # Should be protected
-##        urllib._urlopener = GRUFTestCase.UnauthorizedOpener()
-##        try:
-##            urllib.urlopen, base+'/secret_html'
-##        except Unauthorized:
-##            pass
-
-##    def testAbsoluteURL(self):
-##        '''Test absolute_url'''
-##        self.assertEquals(self.gruf_folder.absolute_url(), base)
+        if getSecurityManager().checkPermission(Permissions.change_permissions, self.sublr2,):
+            self.failUnless(not failed, "Must have the permission here.")
+        else:
+            self.failUnless(failed, "Must NOT have the permission here.")
 
 
+    def test13TestCMFLRBehaviour(self,):
+        """Special test to check that CMF's allowedRolesAndUsers is okay
+        """
+        # Allowed patterns
+        normal_allowed = ['r1', 'r2', 'r3', 'user:group_g1', 'user:u6', 'user:u3']
+        normal_allowed.sort()
+        blocked_allowed = ["r1", "r2", "r3", "user:u3", "user:u6", ]
+        blocked_allowed.sort()
+            
+        # Normal behaviour
+        ob = self.subsublr2
+        allowed = {}
+        for r in rolesForPermissionOn('View', ob):
+            allowed[r] = 1
+        localroles = _mergedLocalRoles(ob)
+        for user, roles in localroles.items():
+            for role in roles:
+                if allowed.has_key(role):
+                    allowed['user:' + user] = 1
+        if allowed.has_key('Owner'):
+            del allowed['Owner']
+        allowed = list(allowed.keys())
+        allowed.sort()
+        self.failUnlessEqual(allowed, normal_allowed)
+
+        # LR-blocking behaviour
+        self.gruf._acquireLocalRoles(self.sublr2, 0)
+        ob = self.subsublr2
+        allowed = {}
+        for r in rolesForPermissionOn('View', ob):
+            allowed[r] = 1
+        localroles = _mergedLocalRoles(ob)
+        for user, roles in localroles.items():
+            for role in roles:
+                if allowed.has_key(role):
+                    allowed['user:' + user] = 1
+        if allowed.has_key('Owner'):
+            del allowed['Owner']
+        allowed = list(allowed.keys())
+        allowed.sort()
+        self.failUnlessEqual(allowed, blocked_allowed)
+
+
+    def test14Allowed(self,):
+        """Test if the allowed() method is working properly.
+        We check the roles on lr, and then on sublr2 after local role blocking tweaking.
+        """
+        u2 = self.gruf.getUser("u2")            # Belongs to group_g1
+        u3 = self.gruf.getUser("u3")
+        u6 = self.gruf.getUser("u6")
+
+        # Positive assertions
+        self.failUnless(u2.allowed(self.lr, ("r3", )))
+        self.failUnless(u3.allowed(self.lr, ("r1", "r3", )))
+        self.failUnless(u6.allowed(self.lr, ("r1", "r2", "r3", )))
+        self.failUnless(u2.allowed(self.subsublr2, ("r1", "r2", "r3", )))
+        self.failUnless(u3.allowed(self.subsublr2, ("r1", "r2", "r3", )))
+        self.failUnless(u6.allowed(self.subsublr2, ("r1", "r2", "r3", )))
+        self.failUnless(u3.allowed(self.subsublr3, ("r1", "r2", "r3", )))
+        self.failUnless(u6.allowed(self.subsublr3, ("r1", "r2", "r3", )))
+
+        # Negative assertions
+        self.failUnless(not u2.allowed(self.lr, ("r1", "r2", )))
+        self.failUnless(not u3.allowed(self.lr, ("r2", )))
+        self.failUnless(not u2.allowed(self.subsublr2, ("r1", "r2", )))
+        self.failUnless(not u2.allowed(self.subsublr3, ("r1", "r2", "r3", )))
+        self.failUnless(not u3.allowed(self.subsublr3, ("r3", )))
+        self.failUnless(not u6.allowed(self.subsublr3, ("r3", )))
+
+
+    def test15user_names(self,):
+        """test15user_names(self,) => check if the user_names() method return user IDS and
+        not user NAMES... This is an inconsistency in Zope's code...
+        """
+        un = self.gruf.user_names()
+        users = [
+            'group_g1', 'group_g2', "group_g3", "group_g4",
+            "group_ng1", "group_ng2", "group_ng3", "group_ng4", "group_ng5",
+            "manager",
+            "u1", "u2", "u3", "u4", "u5", "u6", "u7", "u8", "u9", "u10", "u11",
+            "group_extranet", "group_intranet", "group_compta",
+            ]
+        un.sort()
+        users.sort()
+        for u in users:
+            self.failUnless(u in un, "Invalid users list: '%s' is not in acl_users." % (u,))
+        for u in un:
+            self.failUnless(u in users, "Invalid users list: '%s' is in acl_users but shouldn't be there." % (u,))
+        
+
+def _mergedLocalRoles(object):
+    """Returns a merging of object and its ancestors'
+    __ac_local_roles__.
+    This will call gruf's methods. It's made that may to mimic the
+    usual CMF code."""
+    return object.acl_users._getAllLocalRoles(object)
 
 
 #                                                   #
