@@ -2,7 +2,7 @@
 #                 Test GroupUserFolder                  #
 #                                                       #
 #                                                       #
-# (c)2002 Ingeniweb                                     #
+# (c)2002+ Ingeniweb                                    #
 
 
 import os, sys
@@ -137,9 +137,8 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.failUnless(self.gruf.getUserById("g1", default = None) is None)
 
         # check exception raising & default values
-        try: self.gruf.getUserById("ZORGLUB")
-        except ValueError: pass
-        else: raise "AssertionError", "Should raise"
+        ret = self.gruf.getUserById("ZORGLUB")
+        self.failUnlessEqual(ret, None, "getUserById should return None")
         self.failUnless(self.gruf.getUserById("ZORGLUB", default = "bla") == "bla")
 
     def test_getUserByName(self):
@@ -151,9 +150,8 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.failUnless(self.gruf.getUserByName("group_g1", None).getId() == "group_g1")
 
         # Check exception raising
-        try: self.gruf.getUserByName("ZORGLUB")
-        except ValueError: pass
-        else: raise "AssertionError", "Should raise"
+        ret = self.gruf.getUserByName("ZORGLUB")
+        self.failUnlessEqual(ret, None, "getUserByName should return None")
         self.failUnless(self.gruf.getUserByName("ZORGLUB", default = "bla") == "bla")
 
 
@@ -297,9 +295,10 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.failUnless(grp is None)
 
         # Check raise if user/group not found
-        try: self.gruf.getGroupById("ZORGLUB")
-        except ValueError: pass
-        else: raise "AssertionError", "Should raise"
+        self.failUnlessEqual(
+            self.gruf.getGroupById("ZORGLUB"),
+            None,
+            )
         self.failUnless(self.gruf.getGroupById("ZORGLUB", default = "bla") == "bla")
 
     def test_getGroupByName(self):
@@ -315,9 +314,10 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.failUnless(self.gruf.getGroupByName("u1", default = None) is None)
 
         # Check raise if user/group not found
-        try: self.gruf.getGroupByName("ZORGLUB")
-        except ValueError: pass
-        else: raise "AssertionError", "Should raise"
+        self.failUnlessEqual(
+            self.gruf.getGroupByName("ZORGLUB"),
+            None,
+            )
         self.failUnless(self.gruf.getGroupByName("ZORGLUB", default = "bla") == "bla")
 
 
@@ -329,6 +329,7 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             password = "secret",
             roles = [],
             groups = [],
+            domains = (),
             )
         self.failUnless(self.gruf.getUser("created_user"))
         self.gruf.userFolderAddUser(
@@ -336,6 +337,7 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             password = "secret",
             roles = [],
             groups = [],
+            domains = (),
             )
         self.failUnless(self.gruf.getUser("group_test_prefix"))
         self.failIf(self.gruf.getUser("group_test_prefix").isGroup())
@@ -346,14 +348,29 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             password = "secret2",
             roles = ["r1", ],
             groups = ["g1", ],
+            domains = (),
             )
         self.compareRoles(None, "u1", ['r1',], )
 
+    def test_userFolderUpdateUser(self):
+        self.gruf.userFolderUpdateUser(
+            name = "u5",
+            roles = ["r2", ],
+            )
+        self.compareRoles(None, "u5", ['r1', 'r2',], )
+        self.compareGroups("u5", ['g2', 'g3'], )
+        self.gruf.userFolderUpdateUser(
+            name = "u6",
+            roles = None,
+            )
+        self.compareRoles(None, "u6", ['r1', 'r2', ], )
+
     def test_userFolderDelUsers(self):
         self.gruf.userFolderAddUser(
             name = "created_user",
             password = "secret",
             roles = [],
+            domains = (),
             groups = [],
             )
         self.gruf.userFolderDelUsers(['created_user', ])
@@ -403,6 +420,33 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             self.gruf.getGroupByName("created_group").getAllGroupNames(),
             )
 
+        
+    def test_userFolderUpdateGroup(self):
+        self.gruf.userFolderAddGroup(
+            name = "created_group",
+            roles = [],
+            groups = [],
+            )
+        self.gruf.userFolderUpdateGroup(
+            name = "created_group",
+            roles = ["r1", ],
+            groups = ["group_g1", ],
+            )
+        self.compareRoles(None, "created_group", ['r1',], )
+        self.failUnless(
+            "g1" in self.gruf.getGroupByName("created_group").getAllGroupNames(),
+            self.gruf.getGroupByName("created_group").getAllGroupNames(),
+            )
+        self.gruf.userFolderUpdateGroup(
+            name = "created_group",
+            roles = ["r1", ],
+            groups = None,
+            )
+        self.failUnless(
+            "g1" in self.gruf.getGroupByName("created_group").getAllGroupNames(),
+            self.gruf.getGroupByName("created_group").getAllGroupNames(),
+            )
+
     def test_userFolderDelGroups(self):
         self.gruf.userFolderAddGroup(
             name = "created_group",
@@ -478,6 +522,8 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.gruf.userFolderAddUser(
             name = "created_user",
             password = "secret",
+            domains = (),
+            roles = (),
             groups = [],
             )
         self.gruf.userSetGroups("created_user", ["g1", "g2", ], )
@@ -491,8 +537,13 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             name = "created_user",
             password = "secret",
             groups = ["g2", ],
+            roles = (),
+            domains = (),
+            )
+        self.gruf.userAddGroup(
+            "created_user",
+            "g1",
             )
-        self.gruf.userAddGroup("created_user", "g1", )
         self.compareGroups("created_user", ["g1", "g2", ], )
 
     def test_userRemoveGroup(self):
@@ -502,6 +553,8 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
             name = "created_user",
             password = "secret",
             groups = ["g2", "g1", ],
+            domains = (),
+            roles = (),
             )
         self.gruf.userRemoveGroup("created_user", "g1", )
         self.compareGroups("created_user", ["g2", ], )
@@ -800,6 +853,73 @@ class TestGroupUserFolderAPI(GRUFTestCase.GRUFTestCase, testInterface.TestInterf
         self.failUnless(u.getRealId() == "u1")
 
 
+    # Local roles management
+    def test_acquireLocalRoles(self,):
+        """
+        We block LR acquisition on sublr2.
+        See GRUFTestCase to understand what happens (basically, roles in brackets
+        will be removed from sublr2).
+        """
+        # Initial check
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ("r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", "r2", "r3", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", "r3", )))
+        
+        # Disable LR acquisition on sublr2 and test the stuff
+        self.gruf._acquireLocalRoles(self.sublr2, 0)
+        self.failUnless(self.compareRoles(self.sublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.sublr2, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.sublr2, "u6", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u2", ()))
+        self.failUnless(self.compareRoles(self.subsublr2, "u3", ("r1", "r2", )))
+        self.failUnless(self.compareRoles(self.subsublr2, "u6", ("r1", "r2", )))
+
+
+    def test_isLocalRoleAcquired(self,):
+        self.gruf._acquireLocalRoles(self.sublr2, 0)
+        self.failUnless(not self.gruf.isLocalRoleAcquired(self.sublr2))
+        self.failUnless(self.gruf.isLocalRoleAcquired(self.subsublr2))
+        self.gruf._acquireLocalRoles(self.subsublr2, 0)
+        self.failUnless(not self.gruf.isLocalRoleAcquired(self.sublr2))
+        self.failUnless(not self.gruf.isLocalRoleAcquired(self.subsublr2))
+        self.gruf._acquireLocalRoles(self.sublr2, 1)
+        self.failUnless(self.gruf.isLocalRoleAcquired(self.sublr2))
+        self.failUnless(not self.gruf.isLocalRoleAcquired(self.subsublr2))
+        self.gruf._acquireLocalRoles(self.subsublr2, 1)
+        self.failUnless(self.gruf.isLocalRoleAcquired(self.sublr2))
+        self.failUnless(self.gruf.isLocalRoleAcquired(self.subsublr2))
+
+
+    # Audit checks
+
+    def test_getAllLocalRoles(self):
+        # Allowed patterns
+        normal_allowed = {
+            'group_g1': ['r3', ],
+            'u6': ['r2', 'r3', ],
+            'test_user_1_': ['Owner', ],
+            'u3': ['r2', 'r3', ],
+            }
+        blocked_allowed = {
+            'u6': ['r2', ],
+            'test_user_1_': ['Owner', ],
+            'u3': ['r2', ],
+            }
+
+        # Normal behaviour
+        ob = self.sublr2
+        allowed = self.gruf._getAllLocalRoles(ob)
+        self.failUnlessEqual(allowed, normal_allowed)
+
+        # LR-blocking behaviour
+        ob = self.sublr2
+        self.gruf._acquireLocalRoles(ob, 0)
+        allowed = self.gruf._getAllLocalRoles(ob)
+        self.failUnlessEqual(allowed, blocked_allowed)
+
 
 if __name__ == '__main__':
     framework(descriptions=1, verbosity=1)
